c    "Copyright 2007 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Alejandro Crespo, Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.


c       BDR
c       converts x,y,z of particles (Ps) in PART to a surface
c       generates files "surface.txt" and "surface_index.txt" for viewing in matlab
c       formats.
c
c surface_index.txt:
c
c nWS1,nbedS1,nblockS1     !indices for surface.txt for frame 1
c nWS2,nbedS2,nblockS2     !indices for surface.txt for frame 2
c nWS3,nbedS3,nblockS3     !indices for surface.txt for frame 3
c .. ,  .. , ..
c
c surface.txt:
c       x,y,z      !frame1:   !line#=1          !start of bed surface positions
c       . . .                 !...
c
c       x,y,z                 !line#=nbedS1      !end of bed surface positions
c       x,y,z                 !line#=nbedS1+1    !start of solid wedge surface positions
c       . . .                 !...
c
c       x,y,z                 !line#=nblockS1    !end of solid wedge surface positions
c       x,y,z                 !line#=nblockS1+1  !start of solid wedge surface positions
c       . . .                 !...
c
c       x,y,z                 !line#=nWS1        !end of water surface surface positions
c       x,y,z      !frame2:   !line#=nWS1+1      !start of bed surface positions
c       . . .                 !...
c
c       x,y,z                 !line#=nSW1+nbedS2      !end of bed surface positions
c       x,y,z                 !line#=nSW1+nbedS2+1    !start of solid wedge surface positions
c       . . .                 !...
c
c       Each set of nWS# lines constitutes one frame


      program surfaceProfiler

      parameter(np_max = 270000,i_PART_counter_max=2000, ngage_max=9)
      real xp(np_max),yp(np_max),zp(np_max)
      real up(np_max),vp(np_max),wp(np_max)
      real rhop(np_max),p(np_max),pm(np_max),TEp(np_max)
      real vorty(np_max),pVol(np_max)
      real Time(i_PART_counter_max)
      real DT1(i_PART_counter_max),DT2(i_PART_counter_max)  
      real x_wgage(ngage_max),y_wgage(ngage_max),z_wgage(ngage_max)
      real z_highest(ngage_max),zs(ngage_max)
      real z_sum(10000)

      integer i_periodicOBs(3)
      character supp*4, name_orig*40, name_surface*40, char_temp*20
      character dataDir*1, filename*100
      CHARACTER(LEN=10) :: FMT,FMT1
      CHARACTER(LEN=1)  :: TAB,DQ
      
      common /kernels/ h,h2,fourh2, a2, a24
      
      TAB=CHAR(9)     
      FMT="(A)"
      FMT1="(2A)"
      DQ=CHAR(34)
      
       print*
       write(*,*) '<SPHYSICS>  Copyright (C) <2007>'
       write(*,*) 
     & '<Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, '
       write(*,*) 
     & 'Dr Benedict Rogers, Alejandro Crespo, '
       write(*,*) 
     & 'Muthukumar Narayanaswamy, Dr Shan Zou, and Dr Andrea Panizzo >'
       write(*,*) 'This program comes with ABSOLUTELY NO WARRANTY;    '
       write(*,*) 'This is free software, and you are welcome to      '
       write(*,*) 'redistribute it under conditions stated in         '
       write(*,*) 'the GPL License;                                   '


      print*
      print*,' ---   surfaceProfiler_2D.F  ---'
      print*,' ---   GENERATING SURFACES   ---'
      print*,' ---   Distributed under     ---'
      print*,' ---    the GPL License      ---'
      print*

      
      print*
      print*,'Choose profiling Type : '
      print*,'  1: Surface Profile'
      print*,'  2: Wave Gage at (x,y) location '
      print*,'  3: Choices 1 & 2 '
      read(*,*) i_profileChoice
      write(*,*)i_profileChoice
      
      print*,'Plot Boundary Surfaces? (1=Yes, 0=No)'
      read(*,*) i_boundarySurface
      write(*,*)i_boundarySurface
      
      print*,'Directory name of data'
      read(*,*) dataDir
      write(*,*)dataDir
      
c	---- READ PARAMETERS FROM MATLABIN -----

      filename = dataDir//'/matlabin'
      !print*,'Opening file = ',filename
      open(18,file=filename,status='old')
      
      read(18,*) np
      read(18,*) vlx
      read(18,*) vly		!  0 if 2D
      read(18,*) vlz
      read(18,*) out
      read(18,*) nb
      read(18,*) nbf

      close(18)


      if(np.gt.np_max)then
        print*,'Number of entries in file PART exceeds max value'
        print*,'np.gt.np_max'
        print*,'Adjust np_max = ',np_max
        stop
      endif

      filename = dataDir//'/INDAT'
      open(11,file=filename,status='old')
      !open(11,file='INDAT',status='old')
 
      read(11,*)i_kernel
      read(11,*)i_algorithm
      read(11,*)i_densityFilter  ! Using a shepard filter
      read(11,*)i_viscos
      read(11,*)iBC              !BC
      read(11,*)i_periodicOBs(1)
      read(11,*)i_periodicOBs(2)
      read(11,*)i_periodicOBs(3)
      read(11,*)lattice
      read(11,*)i_EoS
      read(11,*)h_SWL 
      read(11,*)B
      read(11,*)gamma
      read(11,*)coef
      read(11,*)eps   ! epsilon XSPH
      read(11,*)rho0
      read(11,*)viscos_val
      read(11,*)visc_wall
      read(11,*)vlx    !Dimensions of container box
      read(11,*)vly    ! vly=0 for 2D
      read(11,*)vlz
      read(11,*)dx
      read(11,*)dy     ! dy=0  for 2D
      read(11,*)dz
      read(11,*)h
      read(11,*)np	
      read(11,*)nb
      read(11,*)nbf
      read(11,*)ivar_dt
      read(11,*)dt
      read(11,*)tmax     ! End of the run(seconds)
      read(11,*)out      ! output every out seconds
      read(11,*)trec_ini ! Initial output 
      read(11,*)dtrec_det !Detailed recording
      read(11,*)t_sta_det
      read(11,*)t_end_det
      read(11,*)i_restartRun
      
      close(11)
      

      grx = 0.0
      gry = 0.0
      grz = -9.81
c      __________  File ISOSURF

c        open(15,file='ISOSURF_BOUND')
c        open(16,file='ISOSURF_FLUID')

c      __________    Parameters


       nbp1=nb+1


       print*,'dx',dx
       print*,'dy',dy
       print*,'dz',dz
       print*,'h ',h
       print*,'h_SWL ',h_SWL

       h2=h*h
       fourh2=4.*h*h
       two_h = 2.0*h

       pi=4.*atan(1.)
       a1 = 10./(pi*7.)      ! 2D Kernel
       a2 = a1/(h*h)
       aa = a1/(h*h*h)
       a24 = 0.25*a2
       


c      __________ Cartesian Grid for surfaces _____________

       iini= 1        !-2
       ifin= vlx/dx   !+2 ! I assume h=srqt(dx**2+dy**2+dz**2)
       jini= 1        !-2
       jfin= vly/dy   !+2
c      kini=-2
c      kfin=vlz/dz+2
       dzt = dz/50.0    !This makes dz between each surface check much smaller and more accurate
       kini=vlz/dzt   
       kini = kini/2    !Start sampling at the mid_depth
       kfin=vlz/dzt+2
       
       print*
       print*,'Interpolation Grid '
       print*,'iini, ifin ',iini, ifin
       print*,'jini, jfin ',jini, jfin
       print*,'kini, kfin ',kini, kfin
       print*


cc       -- BDR correction --
c       iini=1
c       ifin=vlx/dx-1 ! I assume h=srqt(dx**2+dy**2+dz**2)
c       jini=1   !jini=0
c       jfin=vly/dy-1   !jfin=vly/dy
c       kini=0
c       kfin=vlz/dz+2

c      _______ File MATLABIN_ISO _____________

       open(19,file='matlabin_iso')
        write(19,*) iini,ifin,jini,jfin,kini,kfin,dx,dy,dz,out
       close(19)

       if(1.eq.2)then
         print*,'Data in file matlab_iso'
         write(*,*) 'iini,ifin,jini,jfin,kini,kfin,dx,dy,dz'
         write(*,*) iini,ifin,jini,jfin,kini,kfin,dx,dy,dz
         write(*,*) 'nb,np'
         write(*,*) nb,np
       endif
       print*

       print*,'(To view IPART enter 0 and then 0) '
       print*,'Number of START of PART files to be analyzed= '
       read(*,*) N_start
       write(*,*)N_start
       print*,'Number of FINISH of PART files to be analyzed= '
       read(*,*) N_frames
       write(*,*)N_frames
       if(N_frames.gt.i_PART_counter_max)then
         print*,'Number of entries in file DT exceeds max value'
         print*,'Nframes.gt.i_PART_counter_max'
         print*,'Adjust i_PART_counter_max, i_PART_counter_max = ',
     &            i_PART_counter_max
         stop
       endif

       print*,'Calculate Overtopping (1=Yes, 0=No) '
       read(*,*) i_overtopping
       write(*,*)i_overtopping
       if(i_overtopping.eq.1)then
         print*,'Enter Overtopping Location x (m) '
         read(*,*) x_overtopping
         write(*,*)x_overtopping
       endif

       !--- Open OUTPUT Files ---
       open(20,file='post_Analysis/surface_index.txt',
     &      status='unknown')
       if(i_profileChoice.gt.1)then
         open(21,file='post_Analysis/wavegage.txt',
     &        status='unknown')
       endif
111    format(9e16.8)
112    format(3(' ',i8))
113    format(3(' ',e13.5))
114    format(a24,4(' ',f10.5))
115    format(2(' ',e13.5))

      !-- Set indices for each individual surface ---
      if(i_boundarySurface.eq.1)then
        nbedS = nbeach_stop - nbeach_begin +1
        nblockS = nbedS + (nbS-nbed)
        print*,'nbeach_begin ',nbeach_begin
        print*,'nbeach_stop  ',nbeach_stop
        print*,'nbedS        ',nbedS
        print*,'nblockS      ',nblockS
        print*,'h_SWL        ',h_SWL
      endif

      print*
      rho_ref = 0.5*1000.0     !0.25*1000.0
      pmref=pm(1)/2.0
      print*,'rho_ref      ',rho_ref
      print*,'resolution dzt',dzt
        
      !-- Wave gage locations --
      if(i_profileChoice.gt.1)then
        print*,'Enter Number of Wave Gages'
        read(*,*) ngage
        write(*,*) ngage
        if(ngage.gt.ngage_max)then
          print*,'ERROR'
          print*,'ngage.gt.ngage_max'
          stop
        endif
        print*,'Enter Wave Gage Locations (x,y,z) '
        do i_gage=1,ngage
          read(*,*) x_wgage(i_gage), y_wgage(i_gage), z_wgage(i_gage) 
          print*,'Gage Number ',
     &       i_gage,x_wgage(i_gage), y_wgage(i_gage), z_wgage(i_gage)
          !i_wgage(i) = int(x_wgage(i_gage)/dx)
          !j_wgage(i) = int(y_wgage(i_gage)/dy)
        enddo
        filename=dataDir//'/DT'
        open(19,file=filename,status='old')
        read(19,*)char_temp
        nframes = N_start - 1
        do iframe = 1,N_start-1
          read(19,*,end=301) Time(iframe)
        enddo
        do iframe=N_start,N_frames
           read(19,*,end=301) Time(iframe)
           nframes = nframes + 1
        enddo
301     N_frames=nframes         
        close(19)
        print*,' Corrected N_start, N_frames ',N_start,N_frames
      else
        ngage = 0
      endif


c     %LOOP OVER FRAMES
      ngrab = 0 + N_start - 1
      if(N_frames.eq.0)then
        ngrab = ngrab - 1
      endif    
      do iframe=N_start,N_frames

c       % READ IN THE PART FILE FOR EACH FRAME
        ngrab=ngrab+1
        write(supp,'(i4.4)') ngrab
        !print*,'supp =',supp
        if(iframe.eq.0)then
          name_orig=dataDir//'/IPART'
          name_surface='post_Analysis/surface_0000'
        else
          name_orig=dataDir//'/PART_'//supp
          name_surface='post_Analysis/surface_'//supp
        endif
        
        open(23,file=name_orig,status='old')
        if(i_profileChoice.ne.2)then
          print*, 'iframe, name_orig , name_surface ',
     &             iframe,' ', name_orig, name_surface 
          open(24,file=name_surface)
        else
          print*, 'iframe, name_orig  ', iframe,' ', name_orig 
        endif

c       % READ POSITION, VELOCITY, DENSITY, PRESSURE, MASS AND VORTICITY DATA FOR ALL PARTICLES            
         do i_gage = 1,ngage
           z_highest(i_gage) = -0.
         enddo
         overtoppingVolume = 0.0
         N_overtopping = 0
         npp=0 ! keeps track of number of particles
         do i=1,np
             read(23,*,end=300) xp(i),zp(i),up(i),wp(i),rhop(i),
     +                p(i),pm(i)       !BE CAREFUL HERE!!!
            npp=npp+1
            pVol(i) = pm(i)/rhop(i)           !AND HERE!!!!!!!!
            !pm(i) = rhop(i)*pVol(i)           !AND HERE!!!!!!!!
            !- Search for highest particle at wave gage location -
            if(i.gt.nb.and.ngage.gt.0)then
              do i_gage = 1,ngage
                if(xp(i).gt.x_wgage(i_gage).and.
     &             xp(i).lt.x_wgage(i_gage)+two_h)then
                  if(zp(i).gt.z_wgage(i_gage).and.
     &               zp(i).gt.z_highest(i_gage))then
                    z_highest(i_gage) = zp(i)
                  endif
                endif
              enddo
            endif
            !- Search for overtopping volume -
            if(i.gt.nb)then
              if(xp(i).gt.x_overtopping)then
                overtoppingVolume = overtoppingVolume + pVol(i)
                !overtoppingVolume = overtoppingVolume + 0.0004
                N_overtopping = N_overtopping + 1
                !print*,'i xp, zp ',N_overtopping,i,xp(i),zp(i)
              endif
            endif
         enddo
300     np=npp
        print*,'np = ',np

        close (23)

        iip = -17596   !np/2
        if(iip.gt.0)then
          print*,'iip',iip
          print*,'xp(iip),yp(iip),zp(iip) ',xp(iip),yp(iip),zp(iip)
          print*,'up(iip),vp(iip),wp(iip) ',up(iip),vp(iip),wp(iip)
          print*,'rhop(iip),p(iip),pm(iip)',rhop(iip),p(iip),pm(iip)
        endif

c       --- Write to Data Files for generate surfaces for boundaries ---
        if(i_boundarySurface.eq.1)then
cc         --- Bed Surface ---
c          do ii=nbeach_begin,nbeach_stop
c            write(17,113) xp(ii),yp(ii),zp(ii)
c          enddo
cc         --- Block Surface ---
c          do ii=nbed+1,nbS
c            write(17,113) xp(ii),yp(ii),zp(ii)
c          enddo
          do ii=1,nb
            write(24,113) xp(ii),yp(ii),zp(ii)
          enddo
        endif

        !-- Perform 2 loops, first for surface, second for wave gauges --

        if(i_profileChoice.ne.2)then        
c       --- Water Surface ---
        !iini = ifin/2
        !ifin = iini
        do i=iini,ifin
          xgrid=i*dx
           
          !do j=jini,jfin
            j = 0
            ygrid=j*dy

            print*,'iframe, xgrid, ygrid ',iframe,xgrid,ygrid
            
            k = kfin
            do while(k.gt.kini)    !k=kini,kfin
              k = k - 1

              zgrid=k*dzt

              !print*,'k',k
              !print*,'xgrid,ygrid,zgrid',xgrid,ygrid,zgrid
              !print*,'rho_ref, vmass',rho_ref,vmass
              !read(*,*)

              vmass=0.
              nneigh=0
              up_sum = 0.0
              vp_sum = 0.0
              wp_sum = 0.0
              rhopp_sum = 0.0
              rmo_sum = 0.0
              W_ij_sum = 0.0
              nn = nb
              
              do while(nn.lt.np.and.vmass.lt.rho_ref)     !nn=nb+1,np

                nn = nn + 1
                distx=(xp(nn)-xgrid)
                !disty=(yp(nn)-ygrid)   !3-D only
                distz=(zp(nn)-zgrid)
                dist2=(distx*distx+distz*distz)  !2-D
                !dist2=(distx*distx+disty*disty+distz*distz)  !3-D

                if(dist2.gt.1.e-18.and.dist2.le.fourh2)then
                  nneigh=nneigh+1
                  call kernel(dist2,W_ij)
                  Vj = pVol(nn)   !pm(nn)/rhop(nn)
                  Wij_Volj = W_ij*Vj
                  vmass = vmass +rhop(nn)*Wij_Volj
                  W_ij_sum = W_ab_sum + Wij_Volj
                  up_sum = up_sum + up(nn)*Wij_Volj
                  !vp_sum = vp_sum + vp(nn)*Wij_Volj
                  wp_sum = wp_sum + wp(nn)*Wij_Volj
                  !rhopp_sum = rhopp_sum + rhop(nn)*wab_temp*Vj
                  !rmo_sum = rmo_sum + Wij_Volj
                endif

              enddo
              !print*,'rho_ref, vmass',rho_ref,vmass

              if(vmass.ge.rho_ref)then
                !print*,'vmass reached rho_ref',vmass
                !print*,'i,xgrid,ygrid,zgrid ',xgrid,ygrid,zgrid
                nn = np
                k = kini
              !else
              !  print*,'i,xgrid ',xgrid
              end if
            enddo   !end of: do while(k.gt.kfin)
            if(zgrid.gt.kini*dzt)then
              zout = zgrid
            else
              zout = 0.0
            endif
            write(24,113) xgrid,ygrid,zout
          !enddo   !end of: do j=jini,jfin
        enddo   !end of: do i=iini,ifin
        
        endif   !End of:   if(i_profileChoice.ne.2)then

          
c       --- Wave Gages ---
        do i_gage = 1,ngage  
          xgrid=x_wgage(i_gage)
          ygrid=y_wgage(i_gage)
   
          k = kfin
          do while(k.gt.kini)    !k=kini,kfin
            k = k - 1

            zgrid=k*dzt
   
            vmass = 0.0
            nneigh=0
            rhopp_sum = 0.0
            rmo_sum = 0.0
            z_sum(k) = 0.0
            nn = nb
              
            do while(nn.lt.np.and.vmass.lt.rho_ref)     !nn=nb+1,np

                nn = nn + 1
                distx=(xp(nn)-xgrid)
                !disty=(yp(nn)-ygrid)   !3-D only
                distz=(zp(nn)-zgrid)
                dist2=(distx*distx+distz*distz)  !2-D
                !dist2=(distx*distx+disty*disty+distz*distz)  !3-D

                if(dist2.gt.1.e-18.and.dist2.le.fourh2)then
                  nneigh=nneigh+1
                  
                  call kernel(dist2,W_ij)
                  
                  Vj = pVol(nn)   !pm(nn)/rhop(nn)
                  Wij_Volj = W_ij*Vj
                  vmass = vmass +rhop(nn)*Wij_Volj
                endif

            enddo
            if(vmass.ge.rho_ref)then
              !print*,'vmass reached rho_ref',vmass
              !print*,'i,xgrid,ygrid,zgrid ',xgrid,ygrid,zgrid
              nn = np
              k = kini
            !else
            !  print*,'i,xgrid ',xgrid
            end if
            !print*,'vmass ',vmass
            !print*,'k,xgrid,zgrid ',k,xgrid,zgrid
          enddo   !End of:  do k=kini,kfin
          if(zgrid.gt.kini*dzt)then
            zs(i_gage) = zgrid
          else
            zs(i_gage) = 0.0
          endif
          print*,'iframe, Time, xgrid, zs ',
     &      iframe, Time(iframe), xgrid, zs(i_gage),N_overtopping    
          

         if(1.eq.2)then              
          call underBedCheck(xgrid,ygrid,zgrid,i_bed_status)          
          if(i_bed_status.eq.1.and.nneigh.le.5) then   
            !write(16,*) 10
          else if(i_bed_status.eq.1) then   
           !write(16,*) 10
          else if(nneigh.le.5) then   !if(nneigh.le.10) then
            !write(16,*) 0
            z_sum(k) = vmass
          else
            if (vmass.ge.pmref) then
              z_sum(k) = vmass
              !print*,'k,zgrid,vmass,z_sum(k)',k,zgrid,vmass,z_sum(k)
            else
              z_sum(k) = vmass
              !print*,'k,zgrid,vmass,z_sum(k)',k,zgrid,vmass,z_sum(k)
            endif
          endif
           
c         --- Interpolate surface from summation data ---
          do k=kini,kfin
            zgrid=k*dzt
            if(zgrid.gt.0.8*h0)then
              if(z_sum(k).gt.1.0*pmref.and.z_sum(k+1).lt.1.0*pmref)then
                zs(i_gage) = zgrid + 
     &          ((0.5*pmref - z_sum(k+1))/(z_sum(k) - z_sum(k+1)))
     &          *(((k+1)*dzt) - zgrid)
                !write(17,*)iout*(i_frame - 1)*dt,zs
                print*,'iframe,time,i_gage,zs',
     &                  iframe,Time(iframe),zs(i_gage)
              end if
            end if 
          enddo
              
          
          nn = np
          k = kfin
         endif   !End of:   if(1.eq.2)then
          
        end do   !End of:  do i_gage = 1,ngage
        write(21,*) iframe,Time(iframe),(zs(i_gage),i_gage=1,ngage),
     &                   (z_highest(i_gage),i_gage=1,ngage)
     &                   ,overtoppingVolume
         
        write(*,*) 'Image # complete',iframe
          
          
        !nWS = ngage + nblockS
        !write(18,112)nWS,nbedS,nblockS
       enddo   !end of: do iframe=N_start,Nframes 

100    close(13)
       close(15) 
       close(16)
       if(i_profileChoice.ne.2)close(24)
       
       if(i_profileChoice.gt.1)then
         close(20)
         close(21)
       endif
       end
c
c -------------------------------
c
      subroutine kernel(dist2,W_ij)
      
      common /kernels/ h,h2,fourh2, a2, a24

      if(dist2.gt.h2.and.dist2.le.fourh2)then
        rad = sqrt(dist2)
        qq  = rad/h
        tmq = 2.-qq
        tm2 = tmq*tmq
        tm3 = tm2*tmq
        W_ij = a24*tm3

      else if(dist2.lt.h2.and.dist2.ge.1.e-18)then
        rad = sqrt(dist2)
        qq  = rad/h
        q2  = qq*qq
        q3  = q2*qq
        W_ij = a2*( 1. - 1.5*q2 + 0.75*q3 )
      endif
      
      end subroutine kernel
c
c -------------------------------
c
      subroutine underBedCheck(xg,yg,zg,i_bed_status)
      

      x_beach_start = 2.25
      slope = 0.5

      if(xg.lt.x_beach_start)then
        z_bed = 0.0
      else if(xg.gt.x_beach_start)then
        z_bed = (xg-x_beach_start)*slope
      else
        z_bed = 0.0
      end if

      if(zg.gt.z_bed)then
        i_bed_status = 0
      else
        i_bed_status = 1
      end if
      i_bed_status = 0
      
      end subroutine underBedCheck
