c    "Copyright 2010 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Alejandro Crespo, Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.


      subroutine DomainProcs
      include 'common.2D'
      
      integer grid_commRank
      !integer rankCoords
      logical reorder
      
      !--- Define geometry parameters for partitioning ---
      open(11,file='MPI_container_Limits.txt')
        read(11,*) iCartTop_periodicity
        read(11,*) xb_min,xb_max
        read(11,*) yb_min,yb_max
        read(11,*) zb_min,zb_max
        read(11,*) i_MPI_Adapt, nFP_difference_max,
     &             n_itime_Adapt, n_ini_Adapt
        read(11,*) nbfm
        read(11,*) MPI_CartDims
        read(11,*) MPI_AutoDomainProcs,n_procs_x0,n_procs_z0
      close(11)

      !--- Create Cartesian Topology ---
      vlxMPI = xb_max - xb_min
      vlyMPI = yb_max - yb_min
      vlzMPI = zb_max - zb_min
      if(MPI_CartDims.eq.1.or.n_procs.eq.1)then
        length_ratio_zx = nint(vlzMPI/vlxMPI)
        n_procs_z = 1
        n_procs_x = n_procs
        dims(1) = n_procs_x
        dims(2) = n_procs_z
        ndim = 1
      elseif(MPI_CartDims.eq.2)then
       if(MPI_AutoDomainProcs.eq.1)then
        if(vlzMPI.lt.vlxMPI)then
          length_ratio_zx = nint(vlxMPI/vlzMPI)
          n_procs_z = int(sqrt(real(n_procs/length_ratio_zx))) + 1
        else
          length_ratio_zx = nint(vlzMPI/vlxMPI)
          n_procs_z = int(sqrt(real(n_procs*length_ratio_zx)))
        endif
        if(rank_id.eq.0)then
          print*,'vlxMPI, xb_max, xb_min ',vlxMPI, xb_max, xb_min 
          print*,'vlzMPI, zb_max, zb_min ',vlzMPI, zb_max, zb_min 
          print*,'length_ratio_zx, n_procs_z',length_ratio_zx, n_procs_z
        endif
        n_procs_x = n_procs/n_procs_z
       else
        n_procs_x = n_procs_x0
        n_procs_z = n_procs_z0
       endif
       dims(1) = n_procs_z
       dims(2) = n_procs_x
       ndim = 2
      else
        if(rank_id.eq.0)then
          print*
          print*,'ERROR in SPHYSICS_MPI_2D.f'
          print*,'Invalid value for MPI_CartDims ',MPI_CartDims
          call MPI_ABORT(MPI_COMM_WORLD, rc, ierr)
          stop
        endif
      endif
      if(n_procs.gt.n_procs_max)then
        print*
        print*,'ERROR in SPHYSICS_MPI_2D.f'
        print*,'n_procs.gt.n_procs_max !!!'
        print*,'n_procs     = ',n_procs
        print*,'n_procs_max = ',n_procs_max
        print*,'Modify n_procs_max in common_MPI.2D'
        stop
      endif
      reorder = .false.

      call MPI_CART_CREATE(MPI_COMM_WORLD, ndim, dims, 
     &       iCartTop_periodicity, reorder, grid_comm, ierr)
      call MPI_COMM_RANK  (grid_comm, grid_commRank, ierr)
      call MPI_CART_COORDS(grid_comm, grid_commRank, ndim, 
     &                     rankCoords, ierr)

      if(rank_id.eq.0)then
        print*
        print*,'Created Cartesian Topology'
        print*
        print*,'Number of processes: n_procs = ',n_procs
        if(iCartTop_periodicity(1))then
          print*,'1-D MPI-Periodicity Activated in X-Direction'
        else 
          print*,'1-D MPI-Periodicity NOT Activated in X-Direction'
        end if
        if(iCartTop_periodicity(2))then
          print*,'1-D MPI-Periodicity Activated in Y-Direction'
        else 
          print*,'1-D MPI-Periodicity NOT Activated in Y-Direction'
        end if
        if(iCartTop_periodicity(3))then
          print*,'1-D MPI-Periodicity Activated in Z-Direction'
        else 
          print*,'1-D MPI-Periodicity NOT Activated in Z-Direction'
        end if
        print*
      end if

      if(ndim.eq.1)then
        call MPI_CART_SHIFT(grid_comm, 0, 1, west, east, ierr)
        rank_idx = rankCoords(1)
        rank_idz = rankCoords(2)
c
      elseif(ndim.eq.2)then
       if(n_procs_x.gt.1)then
        call MPI_CART_SHIFT(grid_comm, 1, 1, west, east, ierr)
        call MPI_CART_SHIFT(grid_comm, 0, 1, below, above, ierr)
c
        rank_idx = rankCoords(2)
        rank_idz = rankCoords(1)
c
       elseif(n_procs_x.eq.1)then
        call MPI_CART_SHIFT(grid_comm, 0, 1, west, east, ierr)
        call MPI_CART_SHIFT(grid_comm, 1, 1, below, above, ierr)
        rank_idx = rankCoords(1)
        rank_idz = rankCoords(2)
c
       else
        print*,'ERROR in DomainProcs_MPI_2D.f'
        print*,'n_procs_x.eq.0 !!!'
        print*,'n_procs     = ',n_procs
        print*,'n_procs_max = ',n_procs_max
        stop
       endif
      endif
 
      !- Create variables for calling multi-dimension MPI Topologies -
      !- Z-dir -
      if(ndim.gt.1.and.(iCartTop_periodicity(3).or.n_procs_z.gt.1))then
        i_call_UpDown = 1
      else
        i_call_UpDown = 0
      endif
c     !- Y-dir -
c     if(ndim.eq.3.and.(iCartTop_periodicity(2).or.n_procs_y.gt.1))then
c       i_call_NorthSouth = 1
c     else
c       i_call_NorthSouth = 0
c     endif
       
      if(rank_id.eq.0)then
        print*
        print*,'MPI_CartDims ',MPI_CartDims
        print*,'ndim, dims ',ndim, dims
        print*,'vlxMPI, vlzMPI, length_ratio_zx ',
     &          vlxMPI, vlzMPI, length_ratio_zx
        print*,'n_procs, n_procs_x, n_procs_z ',
     &          n_procs, n_procs_x, n_procs_z
        if(n_procs_x*n_procs_z.lt.n_procs)then
          print*
          print*,'ERROR in SPHYSICS_MPI_2D.f'
          print*,'Invalid value for n_procs '
          print*,'n_procs_x*n_procs_z < n_procs '
          print*,'Try increasing n_procs'
          call MPI_ABORT(MPI_COMM_WORLD, rc, ierr)
          stop
        endif
        print*
      endif
      
      if(rank_id.eq.0)then
        print*
        print*,'Dimension X, dims(1) = ',dims(1)
        if(ndim.eq.2)then
          print*,'Dimension Z, dims(2) = ',dims(2)
        endif
        open(44,file='MPI_DomainProcs.txt',STATUS='replace')
        write(44,483)
     &  'rank_id, rankCoords, rank_idx, rank_idz, '
     &//'east, west, above, below'
         close(44)
      endif
      call MPI_BARRIER(MPI_COMM_WORLD, ierr)   !Synchronise all processes to this point
      if(ndim.eq.1)then
        do i=0,n_procs-1
          if(i.eq.rank_id)then
            open(44,file='MPI_DomainProcs.txt',
     &               STATUS='old',POSITION='append')
            write(44,482)
     &      rank_id, rankCoords, rank_idx, rank_idz, east, west, -1, -1
            close(44)
          end if
          call MPI_BARRIER(MPI_COMM_WORLD, ierr)   !Synchronise all processes to this point
        end do
      elseif(ndim.eq.2)then
        do i=0,n_procs-1
          if(i.eq.rank_id)then
            open(44,file='MPI_DomainProcs.txt',
     &               STATUS='old',POSITION='append')
            write(44,482)
     &      rank_id, rankCoords, rank_idx, rank_idz, 
     &      east, west, above, below
            close(44)
          end if
          call MPI_BARRIER(MPI_COMM_WORLD, ierr)   !Synchronise all processes to this point
        end do
      endif
482   format(9(' ',i6,' '))
483   format(a66)
      !---------------------------------  
      
      return
      end       
