c    "Copyright 2010 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Dr Alejandro Crespo, Dr Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.


       program sph
       
C
c			Axis
c			^
c			|      / Y
c			| Z   /
c			|    /
c			|   /
c			|  /
c			| /
c			|/
c			--------------> X
c

      include 'common.2D'
       
      integer grid_commRank
      !integer rankCoords
      logical reorder

      character supp*4,name*40,detsupp*4,Vortname*40
              
      double precision time_begin, time_end
      REAL time_begin_cpu0, time_end_cpu0
       
      
      !--- Initialise MPI WORLD ---
      call MPI_INIT(ierr)                                  !Starts MPI
      if(ierr.ne.MPI_SUCCESS)then
        print*,'MPI Initialisation error'
        call MPI_ABORT(MPI_COMM_WORLD, rc, ierr)
      end if
      call MPI_COMM_RANK(MPI_COMM_WORLD, rank_id, ierr)    !Get Current Process ID
      call MPI_COMM_SIZE(MPI_COMM_WORLD, n_procs, ierr)    !Get # procs from env or command line

      !CALL CPU_TIME (time_begin)
      if(rank_id.eq.0)then
        time_begin = MPI_Wtime()
      end if

      if(rank_id.eq.0)then
        print*
        write(*,*) '<SPHYSICS>  Copyright (C) <2010>'
        write(*,*) 
     & '<Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, '
        write(*,*) 
     & 'Dr Benedict Rogers, Dr Alejandro Crespo, '
        write(*,*) 
     & 'Dr Muthukumar Narayanaswamy, Dr Shan Zou, & Dr Andrea Panizzo >'
        write(*,*) 'This program comes with ABSOLUTELY NO WARRANTY;    '
        write(*,*) 'This is free software, and you are welcome to      '
        write(*,*) 'redistribute it under conditions stated in         '
        write(*,*) 'the GPL License;                                   '
       
        print*
        print*,' --- parallelSPHYSICS_2D.F   ---'
        print*,' ---   Distributed under     ---'
        print*,' ---    the GPL License      ---'
        print*
      end if
       
      call DomainProcs
      !stop   
c
c  ...  file specifications
c       
      if(rank_id.eq.0)then
       open(50,file='ENERGY')
       
       open(38,file='MPI_Partition_Positions.dat')      
      endif
       
       pi=4.*atan(1.)
       g=9.81
       
c
c  ...  initialization
c      
       call getdata
       !call energy(g)   !Needs MPI_GATHER      
       i_resident_0 = 0  !Resident Fixed Particles
       call ini_divide(1)
       call divide(1,nbf_local,1,i_resident_0)   !3rd argument = kind_p
       call keep_list
       
       if(i_restartRun.eq.1.or.i_restartRun.eq.3) then
         open(19,file='DT',status='old',access='append')
       else
         open(19,file='DT') 
         write(19,*) 'time dt1 dt2 dt_new'
       endif
       close(19)
       
      if(rank_id.eq.0) then
        print*
        write(*,*) 'Before main loop, itime = ',itime
        print*
        CALL CPU_TIME (time_begin_cpu0)
      endif
      !stop
       
c
c  .............................. MAIN LOOP ...................................
c
       
       tdetail=0
       ngrabdet=0
       grab_P=0.0
       grab_E=0.0
       grx=0.0
       gry=0.0
       grz=-g
                    
       do while (time.lt.tmax)
       !do itime_temp = 1,50
          visc_dt=0.
       
          if((time-trec_ini)-out*ngrab.ge.out) then 
            ipoute =1
          else
            ipoute =0
          endif
   

         !-- Call Main Subroutines --  
         call step
                  
         itime=itime+1
         time=time+dt 

         !-- File Output Routines --
         if(ipoute.eq.1) then 
       
           ngrab=ngrab+1
           write(supp,'(i4.4)') ngrab
           name='PART_'//supp
           if(rank_id.eq.0)then
             write(*,*) name
             open(23,file=name,STATUS='replace')
             close(23)           

             if(i_vort.eq.1) then
               Vortname='VORT_'//supp
               open(24,file=Vortname,STATUS='replace')  !pre-Open vorticity File
               close(24)
             endif
           endif   
       
           call poute(22,name,Vortname)
           
           !close(23)           !Now in poute_MPI_2D.f
       
           if(rank_id.eq.0)then
             open(44,file='RESTART')
             write(44,100)itime,time,ngrab,dt
             close(44)
           endif
           
           grab_P=0.
       
         else

           grab_P=grab_P+dt       
         endif

100      format(i8,e16.8,i8,e16.8)
       
         
c        ... In MPI code, densityFilter is called from divide_ParallelPrep
         
         if(grab_E.gt.out.or.time.eq.trec_ini) then
c           call energy(g)
           grab_E=0.
         else
           grab_E=grab_E+dt
         endif   
                                   
c        -- Detailed recording --    
          tdetail=tdetail+dt
       
          if(time.ge.t_sta_det.and.
     +       time.le.t_end_det.and.
     +       tdetail.ge.dtrec_det)then
            tdetail=0
	      ngrabdet=ngrabdet+1
	      write(detsupp,'(i4.4)') ngrabdet

            name='DETPART_'//detsupp
            Vortname='VORT_'//detsupp
            open(24,file=Vortname)  !pre-Open vorticity File
            close(24)
            if(rank_id.eq.0)write(*,*) name
            !open(53,file=name)  !Now in poute_MPI_2D.f
            call poute(52,name,Vortname)
            !close(53)           !Now in poute_MPI_2D.f
          endif
              
       enddo
            
       if(rank_id.eq.0)then
         close(19)
         close(38)
         CALL CPU_TIME (time_end_cpu0)
       end if
       
       name = 'EPART'
       Vortname='VORT_final'
       if(rank_id.eq.0)then
         open(23,file=name,STATUS='replace')
         close(23) 
       
         if(i_vort.eq.1) then
           open(24,file=Vortname,STATUS='replace')  !pre-Open vorticity File
           close(24)
         endif
       endif
       call poute(32,name,Vortname)   !  Final record      
                
       
C	Closing files
       close(22) 
       close(52) 
       close(53) 
       close(50)
       
       
      ! CALL CPU_TIME ( time_end )
      ! write(*,*)'time_begin',time_begin,'seconds'
      ! write(*,*)'time_end',time_end,'seconds'
      ! write(*,*)'Time of operation was',time_end-time_begin,'seconds'
      if(rank_id.eq.0)then
        time_end = MPI_Wtime()
        print*
        print*,'Program finished'
        print*,'n_procs ',n_procs
        print*,'Wall Clock Time (secs)  = ',time_end - time_begin
        print*,'Wall Clock Time (hours) = ',(time_end - time_begin)
     &                                      /3600.0
        write(*,*)'Time of operation was',
     &             time_end_cpu0 - time_begin_cpu0,'seconds'
        print*
      end if
      
      !--- Free-up Derived DataTypes --
      !-- SendRecv_Data.f --
      call MPI_TYPE_FREE(iP_xyzDataTypeA      , ierr)
      call MPI_TYPE_FREE(iP_DataTypeSR        , ierr)
      call MPI_TYPE_FREE(iBP_DataType         , ierr)
      call MPI_TYPE_FREE(iTau_DataType        , ierr)
      call MPI_TYPE_FREE(iInt_IntegerType4    , ierr)
      !-- SendRecv_Data_Conservative.f --
      call MPI_TYPE_FREE(iP_DataTypeSRC       , ierr)
      !-- update_accnSummations.f --
      call MPI_TYPE_FREE(iaccn_DataTypeUA     , ierr)
      !-- update_accnSummations_Conservative.f --
      call MPI_TYPE_FREE(iaccn_DataTypeUAC    , ierr)
      !-- SendRecv_updatedKernelSummations.f --
      call MPI_TYPE_FREE(isumW_DataType       , ierr)
      !-- update_MLS_Summations.f --
      call MPI_TYPE_FREE(iMLS_DataType1       , ierr)
      !-- SendRecv_MLS_Matrices.f --
      call MPI_TYPE_FREE(iMLS_DataType2       , ierr)
      !-- update_KGC_Summations.f --
      call MPI_TYPE_FREE(iKGC_DataType1       , ierr)
      !-- SendRecv_KGC_Matrices.f --
      call MPI_TYPE_FREE(iKGC_DataType2       , ierr)
      !-- index_check.f --
      call MPI_TYPE_FREE(iP_xyzDataTypeB      , ierr)
      call MPI_TYPE_FREE(iP_DataTypeIC        , ierr)
      call MPI_TYPE_FREE(iP_DataTypeV         , ierr)
      call MPI_TYPE_FREE(iInt_IntegerType5    , ierr)
      !-- index_check_Conservative.f --
      call MPI_TYPE_FREE(iP_DataTypeICC       , ierr)
      !-- update_BPInfo_onRoot.f --
      call MPI_TYPE_FREE(iBP_xyz_Data_Type    , ierr)
      !-- rigid_body_motion.f --
      call MPI_TYPE_FREE(irigidBody_Data_Type1, ierr)
      call MPI_TYPE_FREE(irigidBody_Data_Type2, ierr)
      !-- update_vorticitySummations.f --
      call MPI_TYPE_FREE(ivort_DataType       , ierr)


      call MPI_FINALIZE(ierr)
       
       
      write(*,*) 'End, rank_id =  ',rank_id
       
       
       
       
       end       !end main program
       
