c    "Copyright 2010 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Dr Alejandro Crespo, Dr Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.

      subroutine celij(j1,j2,kind_p1,ini_kind_p2,lx2,lz2,id_call)
c
      include 'common.2D'  

      !do kind_p2=ini_kind_p2,2
      !- Changes looping to include BP-BP interactions
      if(kind_p1.eq.1)then
        kind_p2_start = 1
      else
        kind_p2_start = ini_kind_p2
      endif
      
      do kind_p2=kind_p2_start,2
        if(nc(j2,kind_p2).ne.0) then

        do ii=1,nc(j1,kind_p1)
          i = ibox(j1,kind_p1,ii)
         
          do jj=1,nc(j2,kind_p2)
           j = ibox(j2,kind_p2,jj)
           
           drx = xp(i) - xp(j)
           drz = zp(i) - zp(j)

            
           call periodicityCorrection(i,j,drx,drz,lx2,lz2,id_call)


           rr2 = drx*drx + drz*drz

           if(rr2.lt.rr2_min)then
             if(i_particleType(i).lt.4.and.i_particleType(j).lt.4)then
               goto 641
             end if
              print*
              print*,'ERROR In celij.f: rr2.lt.rr2_min'
              print*,'itime ',itime
              print*,'rank_id ',rank_id
              print*,'j1, j2 ',j1, j2
              print*,'kind_p1, kind_p2 ',kind_p1, kind_p2
              print*,'I, j ',I, j
              print*,'xp(I), zp(I) ',xp(i), zp(i)
              print*,'xp(j), zp(j) ',xp(j), zp(j)
              print*,'drx, drz ',drx, drz
              print*,'rr2     ',rr2
              print*,'rr2_min ',rr2_min
              print*,'fourh2 ',fourh2
              print*,'q=sqrt(rr2)/h =',sqrt(rr2)/h
              print*,'i_particleType(i) ',i_particleType(i)
              print*,'i_particleType(j) ',i_particleType(j)
              print*,'pm(I) ',pm(I)
              print*,'pm(j) ',pm(j)
              print*,'up(I), wp(I) ',up(i), wp(i)
              print*,'up(j), wp(j) ',up(j), wp(j)
              print*,'rhop(I), rdot(I) ',rhop(i), rdot(i)
              print*,'rhop(j), rdot(j) ',rhop(j), rdot(j)
              call MPI_BARRIER(MPI_COMM_WORLD, ierr)   !Synchronise all processes to this point
              stop
           end if
           
           if(rr2.lt.fourh2.and.rr2.gt.1.e-18) then
             
             dux = up(i) - up(j)
             duz = wp(i) - wp(j)

             
c            Calculating kernel & Normalized Kernel Gradient
             call kernel(drx,drz,i,j,j1,j2,rr2) 
             call kernel_correction(i,j)
             
c
c            ...  average density
c

             robar  = 0.5*( rhop(i) + rhop(j) )
             one_over_rhobar = 2.0/(rhop(i) + rhop(j))
c
             cbar = 0.5*( cs(i)   + cs(j)   ) 
c
c            ...  inner product rv
c
             dot = drx*dux + dry*duy + drz*duz

c	      Used to calculate the time step due to viscosity

             visc_dt=max(dot/(rr2 + eta2),visc_dt) 
c
c            ...  pressure and viscous force (1992; 3.3)
c
c            pm(j) is mass of particle j
c
             p_v = pr(i) + pr(j)  !+  pi_visc



c           for tensile correction (Monaghan , JCP.  159 (2000) 290- 311)
c           Only to be activated with cubic spline kernel 
c
            if(index_tensile.eq.1) then  

c   _____ Tensile correction

              fab=Wab*od_Wdeltap      !NOTE: We'll use a non-normalized
              fab=fab*fab             !kernel to calculate tensile correction
              fab=fab*fab             !It's the (Wab/Wdeltap)**4  of Monaghan's paper

              if (p(i).gt.0) then
                  Ra= 0.01 *pr(i)
              else
                  Ra= 0.2 *abs(pr(i))
              endif

              if (p(j).gt.0) then
                  Rb= 0.01 *pr(j)
              else
                  Rb= 0.2 *abs(pr(j))
              endif

              p_v = p_v+ (Ra+Rb)*fab

            endif

            if(i_particleType(i).eq.4.and.i_particleType(j).eq.4)then !FP-FP

              ax(i) = ax(i) - pm(j) * p_v * frxi
              az(i) = az(i) - pm(j) * p_v * frzi


              ax(j) = ax(j) + pm(i) * p_v * frxj
              az(j) = az(j) + pm(i) * p_v * frzj


              call gradients_calc(i,j,dux,duz)               
               
              call viscosity(dot,drx,drz,dux,duz,rr2,
     +               cbar,robar,one_over_rhobar,i,j,j1,j2,term2i,term2j)

c  ...  density acceleration  (1992; 3.9)
c       
               
             dot2i = dux*frxi + duz*frzi
             dot2j = dux*frxj + duz*frzj
             ar(i) = ar(i) + pm(j)*dot2i
             ar(j) = ar(j) + pm(i)*dot2j

c  ...  Thermal Energy
c             (Monaghan, JCP 110 (1994) 399- 406)


             term1i=0.5 * p_v *( frxi*dux+frzi*duz)
             term1j=0.5 * p_v *( frxj*dux+frzj*duz)

             aTE(i)=aTE(i)+pm(j) * (term1i+term2i)
             aTE(j)=aTE(j)+pm(i) * (term1j+term2j)

c
c  ...  XSPH correction (Monaghan 1994;  J. Comp. Phys. 110. Formula 2.6)
c
              pmj_Wab_over_rhobar = pm(j)*Wab*one_over_rhobar
              ux(i) = ux(i) - dux*pmj_Wab_over_rhobar  !pm(j) * dux * Wab / robar
              wx(i) = wx(i) - duz*pmj_Wab_over_rhobar  !pm(j) * duz * Wab / robar


              pmi_Wab_over_rhobar = pm(i)*Wab*one_over_rhobar
              ux(j) = ux(j) + dux*pmi_Wab_over_rhobar   !pm(i) * dux * Wab / robar
              wx(j) = wx(j) + duz*pmi_Wab_over_rhobar   !pm(i) * duz * Wab / robar


c  ...  Vorticity calculation

              if(ipoute.eq.1.and.i_vort.eq.1.and.
     &          i_particleType(i).gt.3.and.i_particleType(j).gt.3)then  !FP-FP
                  call vorticity_calc(i,j,dux,duz)
              endif  

            else if(i_particleType(i).eq.4.and.
     &              i_particleType(j).lt.4)then  !FP-BP,   j is boundary particle
              call MonaghanBC(i,j,drx,drz,dot,dux,duz,
     &                        fxbMON,fzbMON)   
              ax(i) = ax(i) + fxbMON
              az(i) = az(i) + fzbMON
              
              temp = pVol(i)*visc_wall*
     &           ((drx*frxi+drz*frzi)/(rr2 + eta2))
              ax(i) = ax(i) + temp*dux
              az(i) = az(i) + temp*duz
              

              if(i_particleType(j).eq.3)then  !Opposite Forces on free-moving object (Not per unit mass, see subroutine whole_body_motion)
                ax(j) = ax(j) - temp*dux*pm(i)  
                az(j) = az(j) - temp*duz*pm(i)  
                
                ax(j) = ax(j) - fxbMON*pm(i)  
                az(j) = az(j) - fzbMON*pm(i)  
              
              end if

            else if(i_particleType(j).eq.4.and.
     &              i_particleType(i).lt.4)then  !FP-BP,   i is boundary particle
              call MonaghanBC(j,i,-drx,-drz,dot,-dux,-duz,
     &                        fxbMON,fzbMON)   
              ax(j) = ax(j) + fxbMON
              az(j) = az(j) + fzbMON
              
              temp = pVol(j)*visc_wall*
     &           ((drx*frxj+drz*frzj)/(rr2 + eta2))
              ax(j) = ax(j) - temp*dux
              az(j) = az(j) - temp*duz

               
              if(i_particleType(i).eq.3)then  !Opposite Forces on free-moving object (Not per unit mass, see subroutine whole_body_motion)
                ax(i) = ax(i) + temp*dux*pm(j)  
                az(i) = az(i) + temp*duz*pm(j)  
                
                ax(i) = ax(i) - fxbMON*pm(j)  
                az(i) = az(i) - fzbMON*pm(j)  
               
              end if

            else if(i_particleType(i).lt.4.and.
     &              i_particleType(j).lt.4)then !BP-BP
              if(i_particleType(i).gt.2)then !i is free-moving BP
                if(i_particleType(j).gt.2)then  !j is free-moving BP
                  if(num_FB.gt.1)then
                    !Loop to prevent BP within a Floating Body interacting with a BP within the same Floating Body
                    i_num_FB = 1
                    i_BP_BP_interaction = 0
                    i_absolute = iabsolute_BP_Pointer_Index(i)
                    j_absolute = iabsolute_BP_Pointer_Index(j)
                    id_num_FB_i = i_FB_Pointer_Info(i_absolute)
                    id_num_FB_j = i_FB_Pointer_Info(j_absolute)
                    do while (i_BP_BP_interaction.lt.1)
                      if(i_num_FB.lt.num_FB)then
                        i_num_FB = i_num_FB + 1
                        if(id_num_FB_i.ne.id_num_FB_j)then
                          call MonaghanBC
     &                         (i,j,drx,drz,dot,dux,duz,
     &                          fxbMON,fzbMON)
                          ax(i) = ax(i) + fxbMON*pm(j)
                          az(i) = az(i) + fzbMON*pm(j)
                          
                          ax(j) = ax(j) - fxbMON*pm(j)
                          az(j) = az(j) - fzbMON*pm(j)
                          
                          i_BP_BP_interaction = 1
                        endif
                      else
                        i_BP_BP_interaction = 1
                      endif
                    enddo
                  endif
                else  !j is prescribed BP
                  call MonaghanBC(i,j,drx,drz,dot,dux,duz,
     &                            fxbMON,fzbMON)
                  ax(i) = ax(i) + fxbMON*pm(j)
                  az(i) = az(i) + fzbMON*pm(j)
                  
                end if    !End of:  if(j.gt.nbfm)then
              else if(i_particleType(j).eq.3)then   !j is free-moving BP, i is prescibed BP
                call MonaghanBC(j,i,-drx,-drz,dot,-dux,-duz,
     &                          fxbMON,fzbMON)
                ax(j) = ax(j) + fxbMON*pm(i)
                az(j) = az(j) + fzbMON*pm(i)
                
              endif   !End of:  if(i.gt.nbfm)then
            endif  ! Interaction fluid-fluid or fluid-boundary
		  
c           -- Debuggin Code --
c           itime_check = 10e8   !1443   !10e8   !0    !10e8   !0    ! 
c           if(rank_id.eq.2.and.itime.ge.itime_check)then
c             if(i.eq.133)then   !if(i.eq.5661)then
c               print*
c               print*,'In CELIJ.f, itime ', itime
c               print*,'j1, j2,id_call ',j1,j2,id_call
c               print*,'I, j ',i,j
c               print*,'xp(I), zp(I) ',xp(i),zp(i)
c               print*,'xp(j), zp(j) ',xp(j),zp(j)
c               print*,'drx , drz ',drx,drz
c               print*,'rr2, fourh2 ',rr2,fourh2
c               print*,'i_particleType(i) ',i_particleType(i)
c               print*,'i_particleType(j) ',i_particleType(j)
c               print*,'pm(I), pm(j) ',pm(I), pm(j)
c               print*,'sum_wab(I), Wab*pVol(j) ',sum_wab(i), Wab*pVol(j)
c               if(i_particleType(i).eq.3.and.i_particleType(j).eq.3)then
c                 !if(i_particleType(i).ne.3)id_num_FB_i = 0
c                 !if(i_particleType(j).ne.3)id_num_FB_j = 0
c                 if(num_FB.gt.1)then
c                   print*,'id_num_FB_i ',id_num_FB_i
c                   print*,'id_num_FB_j ',id_num_FB_j
c                   if(id_num_FB_i.ne.id_num_FB_j)then
c                     print*,'+ fxbMON*pm(j) ', fxbMON*pm(j)
c                     print*,'+ fzbMON*pm(j) ', fzbMON*pm(j)
c                   endif
c                 !else
c                 !  print*,'+ fxbMON*pm(j) ', fxbMON*pm(j)
c                 !  print*,'+ fzbMON*pm(j) ', fzbMON*pm(j)
c                 endif
c               elseif(i_particleType(j).eq.4)then
c                 print*,'- fxbMON*pm(j)                  ',-fxbMON*pm(j)
c                 print*,'- fzbMON*pm(j)                  ',-fzbMON*pm(j)
c                 print*,'- temp*dux*pm(j)/SpecificWeight ',
c     &                   - temp*dux*pm(j)/SpecificWeight
c                 print*,'- temp*duz*pm(j)/SpecificWeight ',
c     &                   - temp*duz*pm(j)/SpecificWeight
c               else
c                 print*,'+ fxbMON*pm(j)                  ',fxbMON*pm(j)
c                 print*,'+ fzbMON*pm(j)                  ',fzbMON*pm(j)
c               endif
c               print*,'ax(I), az(I) ',ax(i),az(i)
c               print*
c             elseif(j.eq.133)then   !elseif(j.eq.5661)then
c               print*
c               print*,'In CELIJ.f, itime ', itime
c               print*,'j1, j2,id_call ',j1,j2,id_call
c               print*,'i, J ',i,j
c               print*,'xp(i), zp(i) ',xp(i),zp(i)
c               print*,'xp(J), zp(J) ',xp(j),zp(J)
c               print*,'-drx,-drz ',-drx,-drz
c               print*,'rr2, fourh2 ',rr2,fourh2
c               print*,'i_particleType(i) ',i_particleType(i)
c               print*,'i_particleType(j) ',i_particleType(j)
c               print*,'pm(i), pm(J) ',pm(i), pm(j)
c               print*,'sum_wab(J), Wab*pVol(i) ',sum_wab(j), Wab*pVol(i)
c               if(i_particleType(i).eq.3.and.i_particleType(j).eq.3)then
c                 !if(i_particleType(i).ne.3)id_num_FB_i = 0
c                 !if(i_particleType(j).ne.3)id_num_FB_j = 0
c                 if(num_FB.gt.1)then
c                   print*,'id_num_FB_i ',id_num_FB_i
c                   print*,'id_num_FB_j ',id_num_FB_j
c                   if(id_num_FB_i.ne.id_num_FB_j)then
c                     print*,'+ fxbMON*pm(i) ', fxbMON*pm(i)
c                     print*,'+ fzbMON*pm(i) ', fzbMON*pm(i)
c                   endif
c                 !else
c                 !  print*,'+ fxbMON*pm(i) ', fxbMON*pm(i)
c                 !  print*,'+ fzbMON*pm(i) ', fzbMON*pm(i)
c                 endif
c               elseif(i_particleType(i).eq.4)then
c                 print*,'- fxbMON*pm(i)                  ',-fxbMON*pm(i)
c                 print*,'- fzbMON*pm(i)                  ',-fzbMON*pm(i)
c                 print*,'- temp*dux*pm(i)/SpecificWeight ',
c     &                   - temp*dux*pm(i)/SpecificWeight
c                 print*,'- temp*duz*pm(i)/SpecificWeight ',
c     &                   - temp*duz*pm(i)/SpecificWeight
c               else
c                 print*,'+ fxbMON*pm(i)                  ',fxbMON*pm(i)
c                 print*,'+ fzbMON*pm(i)                  ',fzbMON*pm(i)
c               endif
c               print*,'ax(J), az(J) ',ax(j),az(j)
c               print*
c             endif           
c           endif
          
           
           endif ! if q<2
          enddo ! Box jj
         enddo ! Box ii
641     endif  ! Box jj is not empty
      enddo   ! Kind of particle

      end